@@ -6,14 +6,41 @@
 #' @param appdir Directory containing the application.
 #' @param destdir Destination directory.
 #' @param subdir Subdirectory of `destdir` to write the app to.
-#' @param verbose Print verbose output. Defaults to `TRUE` if running
-#'    interactively.
+#' @param quiet Suppress console output during export. Follows the global
+#'   `shinylive.quiet` option or defaults to `FALSE` in interactive sessions if
+#'   not set.
+#' @param verbose Deprecated, please use `quiet` instead.
+#' @param wasm_packages Download and include binary WebAssembly packages as
+#'   part of the output app's static assets. Defaults to `TRUE`.
+#' @param package_cache Cache downloaded binary WebAssembly packages. Defaults
+#'   to `TRUE`.
+#' @param assets_version The version of the Shinylive assets to use in the
+#'   exported app. Defaults to [assets_version()]. Note, not all custom assets
+#'   versions may work with this release of \pkg{shinylive}. Please visit the
+#'   [shinylive asset releases](https://github.com/posit-dev/shinylive/releases)
+#'   website to learn more information about the available `assets_version`
+#'   values.
+#' @param template_dir Path to a custom template directory to use when exporting
+#'   the shinylive app. The template can be copied from the shinylive assets
+#'   using: `fs::path(shinylive:::assets_dir(), "export_template")`.
+#' @param template_params A list of parameters to pass to the template. The
+#'   supported parameters depends on the template being used. Custom templates
+#'   may support additional parameters (see `template_dir` for instructions on
+#'   creating a custom template or to find the current shinylive assets' 
+#'   templates).
+#'   
+#'   With shinylive assets > 0.4.1, the default export template supports the
+#'   following parameters:
+#' 
+#'   1. `title`: The title of the app. Defaults to `"Shiny app"`.
+#'   2. `include_in_head`, `include_before_body`, `include_after_body`: Raw
+#'      HTML to be included in the `<head>`, just after the opening `<body>`,
+#'      or just before the closing `</body>` tag, respectively.
 #' @param ... Ignored
 #' @export
 #' @return Nothing. The app is exported to `destdir`. Instructions for serving
 #' the directory are printed to stdout.
-#' @importFrom rlang is_interactive
-#' @examplesIf interactive()
+#' @examplesIf rlang::is_interactive()
 #' app_dir <- system.file("examples", "01_hello", package = "shiny")
 #' out_dir <- tempfile("shinylive-export")
 #'
@@ -25,62 +52,85 @@
 #'   httpuv::runStaticServer(out_dir)
 #' }
 export <- function(
-    appdir,
-    destdir,
-    ...,
-    subdir = "",
-    verbose = is_interactive()) {
-  verbose_print <- if (verbose) message else list
+  appdir,
+  destdir,
+  ...,
+  subdir = "",
+  quiet = getOption("shinylive.quiet", !is_interactive()),
+  wasm_packages = TRUE,
+  package_cache = TRUE,
+  assets_version = NULL,
+  template_dir = NULL,
+  template_params = list(),
+  verbose = NULL
+) {
+  if (!is.null(verbose)) {
+    rlang::warn("The {.var verbose} argument is deprecated. Use {.var quiet} instead.")
+    if (missing(quiet)) {
+      quiet <- !verbose
+    }
+  }
 
-  stopifnot(fs::is_dir(appdir))
+  local_quiet(quiet)
+  cli_alert_info("Exporting Shiny app from: {.path {appdir}}")
+  cli_alert("Destination: {.path {destdir}}")
+
+  if (is.null(assets_version)) {
+    assets_version <- assets_version()
+  }
+
+  if (!fs::is_dir(appdir)) {
+    cli::cli_abort("{.var appdir} must be a directory, but was provided {.path {appdir}}.")
+  }
   if (!(
     fs::file_exists(fs::path(appdir, "app.R")) ||
       fs::file_exists(fs::path(appdir, "server.R"))
   )) {
-    stop("Directory ", appdir, " does not contain an app.R or server.R file.")
+    cli::cli_abort("Directory {.path {appdir}} does not contain an app.R or server.R file.")
   }
 
   if (fs::is_absolute_path(subdir)) {
-    stop(
-      "export(subdir=) was supplied an absolute path (`", subdir, "`).",
-      " Only relative paths are allowed."
+    cli::cli_abort(
+      "{.var subdir} was supplied an absolute path ({.path {subdir}}), but only relative paths are allowed."
     )
   }
 
   if (!fs::dir_exists(destdir)) {
-    verbose_print("Creating ", destdir, "/")
     fs::dir_create(destdir)
   }
 
 
-  cp_funcs <- create_copy_fn(overwrite = FALSE, verbose_print = verbose_print)
+  cp_funcs <- create_copy_fn(overwrite = FALSE)
   mark_file <- cp_funcs$mark_file
   copy_files <- cp_funcs$copy_files
 
-  assets_path <- assets_dir()
+  assets_path <- assets_dir(version = assets_version)
 
   # =========================================================================
   # Copy the base dependencies for shinylive/ distribution. This does not
   # include the R package files.
   # =========================================================================
-  verbose_print(
-    "Copying base Shinylive files from ", assets_path, "/ to ", destdir, "/"
-  )
+  cli_progress_step("Copying base Shinylive files")
+
   # When exporting, we know it is only an R app. So remove python support
-  base_files <- c(shinylive_common_files("base"), shinylive_common_files("r"))
-  if (verbose) {
+  base_files <- c(
+    shinylive_common_files("base", version = assets_version),
+    shinylive_common_files("r", version = assets_version)
+  )
+
+  if (!is_quiet()) {
     p <- progress::progress_bar$new(
-      format = "[:bar] :percent",
+      format = "[:bar] :percent\n",
       total = length(base_files),
-      clear = TRUE,
-      # show_after = 0
+      clear = FALSE,
+      show_after = 0
     )
   }
   Map(
     file.path(assets_path, base_files),
     file.path(destdir, base_files),
     f = function(src_path, dest_path) {
-      if (verbose) {
+      if (!is_quiet()) {
         p$tick()
       }
       mark_file(src_path, dest_path)
@@ -98,6 +148,7 @@ export <- function(
   # })
   # Copy all files in one call
   copy_files()
+  cli_progress_done()
 
   # =========================================================================
   # Load each app's contents into a list[FileContentJson]
@@ -139,6 +190,13 @@ export <- function(
 
   #     copy_fn(src_path, dest_path)
 
+  # =========================================================================
+  # Copy app package dependencies as Wasm binaries
+  # =========================================================================
+  if (wasm_packages) {
+    download_wasm_packages(appdir, destdir, package_cache)
+  }
+
   # =========================================================================
   # For each app, write the index.html, edit/index.html, and app.json in
   # destdir/subdir.
@@ -146,14 +204,17 @@ export <- function(
   write_app_json(
     app_info,
     destdir,
-    html_source_dir = fs::path(assets_path, "export_template"),
-    verbose = verbose
+    template_dir = template_dir %||% fs::path(assets_path, "export_template"),
+    quiet = quiet,
+    template_params = template_params
   )
 
-  verbose_print(
-    "\nRun the following in an R session to serve the app:\n",
-    "  httpuv::runStaticServer(\"", destdir, "\")\n"
-  )
+  # Escape backslashes in destdir because Windows
+  destdir_esc <- gsub("\\\\", "\\\\\\\\", destdir)
 
-  invisible()
+  cli_alert_success("Shinylive app export complete.")
+  cli_alert_info("Run the following in an R session to serve the app:")
+  cli_text('{.run httpuv::runStaticServer("{destdir_esc}")}')
+
+  invisible(destdir)
 }
